/*
 * Filter
 * Copyright (C) 2011 by Apaq
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package dk.apaq.framework.criteria.jpa.interpreters;

import dk.apaq.framework.criteria.jpa.CriteriaForJpa.WhereClause;
import dk.apaq.framework.criteria.rules.LikeRule;

/**
 * Interpreter for Like rule.
 */
public class LikeRuleInterpreter extends AbstractInterpreter<LikeRule> {

    private static final char ESCAPECHAR = '!';

    public boolean interpret(WhereClause clause, LikeRule rule) {
        if (rule.getValue() == null) {
            return false;
        }

        String searchString = rule.getValue();

        //percentage is wildcard and underscore singlewildcard
        searchString = searchString.replace("%", ESCAPECHAR + "%");
        searchString = searchString.replace("_", ESCAPECHAR + "_");
        searchString = searchString.replace('*', '%');
        searchString = searchString.replace('?', '_');

        String paramName = getParamName();
        StringBuilder sb = new StringBuilder();

        if (!rule.isCaseSensitive()) {
            sb.append("UPPER(e.").append(rule.getPropertyId()).append(")");
        } else {
            sb.append("e.").append(rule.getPropertyId());
        }

        sb.append(" LIKE ");

        if (!rule.isCaseSensitive()) {
            sb.append("UPPER(:").append(paramName).append(")");
        } else {
            sb.append(":").append(paramName);
        }

        sb.append(" ESCAPE '").append(ESCAPECHAR).append("'");

        String jpql = sb.toString();
        clause.appendStatement(jpql);
        clause.appendParameter(paramName, searchString);
        return true;
    }
}
