
package dk.apaq.framework.criteria.simplequery;

import java.util.HashMap;
import java.util.Map;

import dk.apaq.framework.criteria.Rule;
import dk.apaq.framework.criteria.rules.AndRule;
import dk.apaq.framework.criteria.rules.CompareRule;
import dk.apaq.framework.criteria.rules.InRule;
import dk.apaq.framework.criteria.rules.IsNullRule;
import dk.apaq.framework.criteria.rules.LikeRule;
import dk.apaq.framework.criteria.rules.NotRule;
import dk.apaq.framework.criteria.rules.OrRule;
import dk.apaq.framework.criteria.simplequery.interpreters.CompareRuleInterpreter;
import dk.apaq.framework.criteria.simplequery.interpreters.InRuleInterpreter;
import dk.apaq.framework.criteria.simplequery.interpreters.Interpreter;
import dk.apaq.framework.criteria.simplequery.interpreters.IsNullRuleInterpreter;
import dk.apaq.framework.criteria.simplequery.interpreters.LikeRuleInterpreter;


/**
 * A composer for SimpleQuery. <br>
 * This composer takes a <code>Rule</code> and translates it to a SimpleQuery String which can
 * be parsed by the <code>SimpleQueryParser</code>.
 * 
 */
public class SimpleQueryComposer {
    private final Map<Class<? extends Rule>, Interpreter<? extends Rule>> interpreterMap =
            new HashMap<Class<? extends Rule>, Interpreter<? extends Rule>>();

    public SimpleQueryComposer() {
        //interpreterMap.put(BetweenRule.class, );
        interpreterMap.put(CompareRule.class, new CompareRuleInterpreter());
        //interpreterMap.put(ContainsRule.class, new ContainsRuleInterpreter());
        interpreterMap.put(InRule.class, new InRuleInterpreter());
        interpreterMap.put(IsNullRule.class, new IsNullRuleInterpreter());
        interpreterMap.put(LikeRule.class, new LikeRuleInterpreter());
        
        
    }
    
    public String compose(Rule rule) {
        StringBuilder query = new StringBuilder();
        buildWhereClause(query, rule);
        return query.toString();
    }
    
    private boolean buildWhereClause(StringBuilder query, Rule filter) {
        if (filter == null) {
            return false;
        }

        //First try an interpreter
        Interpreter interpreter = interpreterMap.get(filter.getClass());
        if (interpreter != null) {
            return interpreter.interpret(query, filter);
        }

        //..else try our special handlers for filtercontainers.
        if (filter instanceof AndRule) {
            int currentRuleStartIndex = query.length();
            int count = 0;
            AndRule af = (AndRule) filter;

            boolean hasAddedRule = false;
            for (Rule currentRule : af.getRules()) {
                int startIndex = query.length();

                boolean statementAppended = buildWhereClause(query, currentRule);

                if (statementAppended) {
                    if (hasAddedRule) {
                        query.insert(startIndex, " AND ");
                    }
                    hasAddedRule = true;
                    count++;
                }
            }

            if (hasAddedRule && count > 1) {
                query.insert(currentRuleStartIndex, "(");
                query.append(")");
            }

            return hasAddedRule;
        }

        if (filter instanceof OrRule) {
            int currentRuleStartIndex = query.length();
            int count = 0;
            OrRule of = (OrRule) filter;
            boolean hasAddedRule = false;

            for (Rule currentRule : of.getRules()) {
                int startIndex = query.length();

                boolean statementAppended = buildWhereClause(query, currentRule);

                if (statementAppended) {
                    if (hasAddedRule) {
                        query.insert(startIndex, " OR ");
                    }
                    hasAddedRule = true;
                    count++;
                }
            }

            if (hasAddedRule && count > 1) {
                query.insert(currentRuleStartIndex, "(");
                query.append(")");
            }

            return hasAddedRule;
        }

        if (filter instanceof NotRule) {
            NotRule af = (NotRule) filter;

            int startIndex = query.length();
            boolean statementAppended = buildWhereClause(query, af.getRuleElement());
            if (statementAppended) {
                query.insert(startIndex, "NOT (");
                query.append(")");
                return true;
            } else {
                return false;
            }
        }

        return false;
    }
}
