package dk.apaq.framework.criteria.simplequery.parser;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import dk.apaq.framework.criteria.Rules;
import dk.apaq.framework.criteria.rules.AndRule;
import dk.apaq.framework.criteria.rules.OrRule;
import org.parboiled.BaseParser;
import org.parboiled.Context;
import org.parboiled.Rule;
import org.parboiled.common.StringUtils;
import org.parboiled.errors.InvalidInputError;
import org.parboiled.support.MatcherPath;
import org.parboiled.support.Var;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Javadoc
 */
public class QueryParser extends BaseParser<Object> {

    public static final String IN = "IN";
    public static final String LITERAL_OPEN_SIMPLE = "'";
    public static final String LITERAL_CLOSE_SIMPLE = "'";
    public static final String LITERAL_OPEN_DOUBLE = "\"";
    public static final String LITERAL_CLOSE_DOUBLE = "\"";
    public static final String NULL = "NULL";
    public static final String TO = "TO";
    public static final String CONTAINS = "CONTAINS";
    public static final String FALSE = "FALSE";
    public static final String TRUE = "TRUE";
    public static final String AND = "AND";
    public static final String COMMA = ",";
    public static final String CURLY_BRACKET_CLOSE = "}";
    public static final String CURLY_BRACKET_OPEN = "{";
    public static final String EQUALS = "=";
    public static final String EQUALS_IGNORE_CASE_AND_ACCENT = "~";
    public static final String GREATER = ">";
    public static final String GREATER_OR_EQUALS = ">=";
    public static final String LESS = "<";
    public static final String LESS_OR_EQUALS = "<=";
    public static final String LIKE = "LIKE";
    public static final String NOT = "NOT";
    public static final String NOT_EQUALS = "!=";
    public static final String NOT_EQUALS_IGNORE_CASE_AND_ACCENT = "!~";
    public static final String OR = "OR";
    public static final String BRACKET_CLOSE = ")";
    public static final String BRACKET_OPEN = "(";
    public static final String SQUARE_BRACKET_CLOSE = "]";
    public static final String SQUARE_BRACKET_OPEN = "[";
    private static final Logger LOG = LoggerFactory.getLogger(QueryParser.class);
    public Rule icNOT = IgnoreCase(NOT);
    public Rule icAND = IgnoreCase(AND);
    public Rule icOR = IgnoreCase(OR);
    public Rule icIN = IgnoreCase(IN);
    public Rule icTO = IgnoreCase(TO);
    public Rule icCONTAINS = IgnoreCase(CONTAINS);
    public Rule icFALSE = IgnoreCase(FALSE);
    public Rule icTRUE = IgnoreCase(TRUE);
    public Rule icLIKE = IgnoreCase(LIKE);
    public Rule icNULL = IgnoreCase(NULL);
    protected Map<String, Class> enums = new LinkedHashMap<String, Class>();

    public Map<java.lang.String, Class> getEnums() {
        return enums;
    }

    public void setEnums(Map<java.lang.String, Class> enums) {
        this.enums = enums;
    }

    /**
     *
     * Remove quote at beginning and ending of String in parameter if necessary <li>"toto" return toto <li>"toto return "toto <li> toto return
     * toto" <li> to"to return to"to
     *
     */
    protected String removeQuote(String s) {
        if (s == null) {
            return s;
        }

        String result = s;
        if ((s.startsWith(LITERAL_OPEN_SIMPLE) || s.startsWith(LITERAL_OPEN_DOUBLE))
                && (s.endsWith(LITERAL_CLOSE_SIMPLE) || s.endsWith(LITERAL_CLOSE_DOUBLE))) {
            result = StringUtils.substring(s, 1, -1);
        }
        return result;
    }

    public Rule start() {
        return Sequence(FirstOf(or(), empty()), push(new AndRule((dk.apaq.framework.criteria.Rule) pop())),
                space(), EOI);
    }

    Rule empty() {
        return Sequence(EMPTY, push(new AndRule()));
    }

    Rule or() {
        return Sequence(and(), ZeroOrMore(space(), icOR, space(), and(),
                push(new OrRule((dk.apaq.framework.criteria.Rule) pop(1), (dk.apaq.framework.criteria.Rule) pop()))));
    }

    Rule and() {
        return Sequence(term(), ZeroOrMore(
                // when no AND or OR is used, AND is default
                // don't change order of FirstOf, this order is important
                FirstOf(Sequence(space(), icAND, space()), Sequence(space(), TestNot(icOR))),
                term(),
                push(new AndRule((dk.apaq.framework.criteria.Rule) pop(1), (dk.apaq.framework.criteria.Rule) pop()))));
    }

    Rule term() {
        return FirstOf(condition(), parens());
    }

    Rule parens() {
        return Sequence(space(), BRACKET_OPEN, space(), or(), space(), BRACKET_CLOSE, space());
    }

    Rule condition() {
        return FirstOf(enumEq(),
                not(), isNull(), isNotNull(),
                greatereq(), lesseq(),
                between(), in(), contains(),/*containsAll(), containsOne(),*/
                eq(), /*neq(),*/ /*eqIgnoreCaseAndAccent(), neqIgnoreCaseAndAccent(),*/
                less(), greater(), like(),/* notlike(),*/
                rTrue(), rFalse()/*, keyword()*/);
    }

    Rule not() {
        return Sequence(space(), icNOT, space(), term(),
                push(Rules.not((dk.apaq.framework.criteria.Rule) pop())));
    }

    Rule isNull() {
        return Sequence(field(), push(match()), space(), EQUALS, space(), icNULL,
                push(Rules.isNull(pop().toString())));
    }

    Rule isNotNull() {
        return Sequence(field(), push(match()), space(), NOT_EQUALS, space(), icNULL,
                push(Rules.not(Rules.isNull(pop().toString()))));
    }

    Rule eq() {
        return Sequence(field(), push(match()), space(), EQUALS, space(), value(),
                push(Rules.equals(pop(1).toString(), pop())));
    }

    Rule neq() {
        return Sequence(field(), push(match()), space(), NOT_EQUALS, space(), value(),
                push(Rules.not(Rules.equals(pop(1).toString(), pop()))));
    }

    Rule enumEq() {
        return Sequence(enumField(), push(match()), space(), EQUALS, space(), valueText(),
                push(Rules.equals(peek(1).toString(), enumFromKeyAndValue(pop(1).toString(), pop().toString()))));
    }

    /**
     *
     * gere eq, startsWith, endsWith, isNull
     *
     * @return
     *
     */
    /*Rule eqIgnoreCaseAndAccent() {

     return Sequence(field(), push(match()), space(), EQUALS_IGNORE_CASE_AND_ACCENT, space(), value(),
     push(new Equals(toElement(pop(1).toString()), (ConditionValue) pop(), true)));

     }
     */
    /**
     *
     * gere eq, isNull
     *
     * @return
     *
     */
    /* Rule neqIgnoreCaseAndAccent() {

     return Sequence(field(), push(match()), space(), NOT_EQUALS_IGNORE_CASE_AND_ACCENT, space(), value(),
     push(new NotEquals(toElement(pop(1).toString()), (ConditionValue) pop(), true)));

     }*/
    Rule less() {
        return Sequence(field(), push(match()), space(), LESS, space(), value(),
                push(Rules.lessThan(pop(1).toString(), pop())));
    }

    Rule lesseq() {
        return Sequence(field(), push(match()), space(), LESS_OR_EQUALS, space(), value(),
                push(Rules.lessOrEqual(pop(1).toString(), pop())));
    }

    Rule greater() {
        return Sequence(field(), push(match()), space(), GREATER, space(), value(),
                push(Rules.greaterThan(pop(1).toString(), pop())));
    }

    Rule greatereq() {
        return Sequence(field(), push(match()), space(), GREATER_OR_EQUALS, space(), value(),
                push(Rules.greaterOrEqual(pop(1).toString(), pop())));
    }

    Rule like() {
        return Sequence(field(), push(match()), space(), icLIKE, space(), value(),
                push(Rules.like(pop(1).toString(), (String) pop(), false)));
    }

    /*Rule notlike() {

        return Sequence(field(), push(match()), space(), icUNLIKE, space(), value(),
                push(Rules.not(Rules.like(pop(1).toString(), (String) pop()))));

    }*/

    Rule between() {
        return Sequence(field(), push(match()), space(), EQUALS, space(), SQUARE_BRACKET_OPEN, space(),
                value(), space(), icTO, space(),
                value(), space(), SQUARE_BRACKET_CLOSE,
                push(Rules.between(pop(2).toString(), pop(1), pop())));
    }

    Rule in() {
        Var<List> elems = new Var<List>(new LinkedList());
        return Sequence(field(), push(match()), space(), icIN, space(),
                BRACKET_OPEN,
                space(), value(), elems.get().add(pop()), space(),
                ZeroOrMore(COMMA, space(), value(), elems.get().add(pop()), space()),
                BRACKET_CLOSE,
                push(Rules.in(pop().toString(), elems.get())));

    }

    Rule contains() {
        return Sequence(field(), push(match()), space(), icCONTAINS, space(), value(),
                push(Rules.contains(pop(1).toString(), pop())));

    }

    /*Rule containsAll() {

     Var<List<ConditionValue>> elems = new Var<List<ConditionValue>>(new LinkedList<ConditionValue>());

     return Sequence(field(), push(match()), space(), EQUALS, space(), SQUARE_BRACKET_OPEN, space(),
     value(), elems.get().add((ConditionValue) pop()), space(), ZeroOrMore(COMMA, space(),
     value(), elems.get().add((ConditionValue) pop()), space()), SQUARE_BRACKET_CLOSE,
     push(new ContainsAll(toElement(pop().toString()), elems.get())));

     }

     Rule containsOne() {

     Var<List<ConditionValue>> elems = new Var<List<ConditionValue>>(new LinkedList<ConditionValue>());

     return Sequence(field(), push(match()), space(), EQUALS, space(), CURLY_BRACKET_OPEN, space(),
     value(), elems.get().add((ConditionValue) pop()), space(), ZeroOrMore(COMMA, space(),
     value(), elems.get().add((ConditionValue) pop()), space()), CURLY_BRACKET_CLOSE,
     push(new ContainsOne(toElement(pop().toString()), elems.get())));

     }

     */
    Rule enumField() {
        //TODO Clean up this mess

        //Pull the keys and sort them with the longest ones first.
        //That makes parboild happy.
        List<String> keys = new ArrayList<String>(enums.keySet());
        Collections.sort(keys, new Comparator<String>() {
            public int compare(java.lang.String o1, java.lang.String o2) {
                return o1.length() < o2.length() ? 1 : -1;
            }
        });

        //Create the rules.
        List<Rule> propRules = new ArrayList<Rule>();
        for (String prop : keys) {
            propRules.add(String(prop));
        }

        //If not rules created then return empty.
        if (propRules.isEmpty()) {
            return NOTHING;
        }

        //Return a rule bases based on the list of rules.
        return FirstOf(propRules.toArray(new Rule[propRules.size()]));
    }

    Rule field() {
        return OneOrMore(FirstOf(CharRange('0', '9'), CharRange('a', 'z'), CharRange('A', 'Z'), AnyOf("_.*")));
    }

    Rule rTrue() {
        return Sequence(icTRUE, push(Boolean.TRUE));
    }

    Rule rFalse() {
        return Sequence(icFALSE, push(Boolean.FALSE));
    }

    Rule rInteger() {
        return Sequence(OneOrMore(CharRange('0', '9')), push(Integer.parseInt(match())));
    }

    Rule value() {
        return FirstOf(rTrue(), rFalse(), rInteger(), valueText());
    }

    Rule valueText() {
        return Sequence(FirstOf(StringLiteralDouble(), StringLiteralSimple(), SimpleString()),
                push(removeQuote(match())));
    }

    Rule SimpleString() {
        return OneOrMore(FirstOf(
                Escape(),
                Sequence(TestNot(AnyOf(" #\t\r\n\\"
                + COMMA
                + LITERAL_OPEN_SIMPLE + LITERAL_OPEN_DOUBLE
                + BRACKET_OPEN + BRACKET_CLOSE
                + CURLY_BRACKET_OPEN + CURLY_BRACKET_CLOSE
                + SQUARE_BRACKET_OPEN + SQUARE_BRACKET_CLOSE)), ANY))).suppressSubnodes();
    }

    Rule StringLiteralSimple() {
        return Sequence(
                LITERAL_OPEN_SIMPLE,
                ZeroOrMore(
                FirstOf(
                EscapeSimple(),
                Sequence(TestNot(AnyOf("\r\n\\" + LITERAL_CLOSE_SIMPLE)), ANY))).suppressSubnodes(),
                LITERAL_CLOSE_SIMPLE);
    }

    Rule StringLiteralDouble() {
        return Sequence(
                LITERAL_OPEN_DOUBLE,
                ZeroOrMore(
                FirstOf(
                EscapeDouble(),
                Sequence(TestNot(AnyOf("\r\n\\" + LITERAL_CLOSE_DOUBLE)), ANY))).suppressSubnodes(),
                LITERAL_CLOSE_DOUBLE);
    }

    Rule Escape() {
        return Sequence('\\', FirstOf(AnyOf("btnfr\'\\"
                + LITERAL_OPEN_SIMPLE + LITERAL_CLOSE_SIMPLE + LITERAL_OPEN_DOUBLE + LITERAL_CLOSE_DOUBLE),
                OctalEscape(), UnicodeEscape()));
    }

    Rule EscapeSimple() {
        return Sequence('\\', FirstOf(AnyOf("btnfr\'\\" + LITERAL_CLOSE_SIMPLE),
                OctalEscape(), UnicodeEscape()));
    }

    Rule EscapeDouble() {
        return Sequence('\\', FirstOf(AnyOf("btnfr\'\\" + LITERAL_CLOSE_DOUBLE),
                OctalEscape(), UnicodeEscape()));
    }

    Rule OctalEscape() {
        return FirstOf(
                Sequence(CharRange('0', '3'), CharRange('0', '7'), CharRange('0', '7')),
                Sequence(CharRange('0', '7'), CharRange('0', '7')),
                CharRange('0', '7'));
    }

    Rule UnicodeEscape() {
        return Sequence(OneOrMore('u'), HexDigit(), HexDigit(), HexDigit(), HexDigit());
    }

    Rule HexDigit() {
        return FirstOf(CharRange('a', 'f'), CharRange('A', 'F'), CharRange('0', '9'));
    }

    Rule space() {
        return ZeroOrMore(AnyOf(" \t\f"));
    }

    protected Enum enumFromKeyAndValue(String key, String value) {
        LOG.debug("Retrieving enum [key={};value={}]", key, value);
        Class enumClass = enums.get(key);
        Enum[] enumConstants = (Enum[]) enumClass.getEnumConstants();
        for (Enum e : enumConstants) {
            if (e.name().equals(value)) {
                return e;
            }
        }

        Context c = getContext();
        String msg = "value for property '" + key + "' is expected to be one of " + Arrays.toString(enumConstants)
                + " but was '" + value + "'.";
        List<MatcherPath> matchers = Arrays.asList(new MatcherPath[]{c.getPath()});
        c.getParseErrors().add(new InvalidInputError(c.getInputBuffer(), c.getStartIndex(), matchers, msg));
        return null;
    }
}
