package dk.apaq.framework.criteria.simplequery;

import dk.apaq.framework.criteria.Rule;
import dk.apaq.framework.criteria.rules.AndRule;
import dk.apaq.framework.criteria.rules.BetweenRule;
import dk.apaq.framework.criteria.rules.CompareRule;
import dk.apaq.framework.criteria.rules.InRule;
import dk.apaq.framework.criteria.rules.LikeRule;
import static org.junit.Assert.*;
import org.junit.Test;

/**
 *
 */
public class SimpleQueryParserTest {

    @Test
    public void testEmpty() {
        String query = "";
        Rule rule = new SimpleQueryParser().parse(query);
        assertNotNull(rule);
        assertTrue(rule instanceof AndRule);
        assertEquals(0, ((AndRule) rule).getRules().size());
    }

    @Test
    public void testCompare() {
        String query = "name=john";
        Rule rule = new SimpleQueryParser().parse(query);
        assertNotNull(rule);
        assertTrue(rule instanceof AndRule);
        
        AndRule ar = (AndRule) rule;
        assertEquals(1, ar.getRules().size());
        
        CompareRule cr = (CompareRule) ar.getRules().get(0);
        assertEquals("name", cr.getPropertyId());
        assertEquals("john", cr.getValue());
    }
    
    @Test
    public void testLike() {
        String query = "name like joh*";
        Rule rule = new SimpleQueryParser().parse(query);
        assertNotNull(rule);
        assertTrue(rule instanceof AndRule);
        
        AndRule ar = (AndRule) rule;
        assertEquals(1, ar.getRules().size());
        
        LikeRule cr = (LikeRule) ar.getRules().get(0);
        assertEquals("name", cr.getPropertyId());
        assertEquals("joh*", cr.getValue());
    }
    
    @Test
    public void testBetween() {
        String query = "age=[25 to 35]";
        Rule rule = new SimpleQueryParser().parse(query);
        assertNotNull(rule);
        assertTrue(rule instanceof AndRule);
        
        AndRule ar = (AndRule) rule;
        assertEquals(1, ar.getRules().size());
        
        BetweenRule cr = (BetweenRule) ar.getRules().get(0);
        assertEquals("age", cr.getPropertyId());
        assertEquals(25, cr.getStartValue());
        assertEquals(35, cr.getEndValue());
    }
    
    @Test
    public void testIn() {
        String query = "age in(25,26,28)";
        Rule rule = new SimpleQueryParser().parse(query);
        assertNotNull(rule);
        assertTrue(rule instanceof AndRule);
        
        AndRule ar = (AndRule) rule;
        assertEquals(1, ar.getRules().size());
        
        InRule cr = (InRule) ar.getRules().get(0);
        assertEquals("age", cr.getPropertyId());
        assertEquals(3, cr.getValues().size());
        assertEquals(25, cr.getValues().get(0));
        assertEquals(26, cr.getValues().get(1));
        assertEquals(28, cr.getValues().get(2));
        
    }
    
    @Test
    public void testInvalid() {
        String query = "age =55 and aadds#€##€€ go go yam";
        try {
            Rule rule = new SimpleQueryParser().parse(query);
            fail("Should have failed");
        } catch(QueryException ex) {
            
        }
        
        
    }
}
