/*
 * Apaq Framework
 * Copyright (C) 2011-2013 by Apaq
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package dk.apaq.framework.criteria;

import java.util.List;
import dk.apaq.framework.criteria.rules.AndRule;
import dk.apaq.framework.criteria.rules.BetweenRule;
import dk.apaq.framework.criteria.rules.CompareRule;
import dk.apaq.framework.criteria.rules.ContainsRule;
import dk.apaq.framework.criteria.rules.InRule;
import dk.apaq.framework.criteria.rules.IsNullRule;
import dk.apaq.framework.criteria.rules.LikeRule;
import dk.apaq.framework.criteria.rules.NotRule;
import dk.apaq.framework.criteria.rules.OrRule;

/**
 * A class with static methods that gives easy access to all basic rules.
 */
public class Rules {

    public static Rule and(Rule ... filters) {
        return new AndRule(filters);
    }
    
    public static Rule or(Rule ... filters) {
        return new OrRule(filters);
    }
    
    public static Rule not(Rule filter) {
        return new NotRule(filter);
    }
    
    public static Rule not(String propertyId, Object value) {
        return new NotRule(new CompareRule(propertyId, value, CompareRule.CompareType.Equals));
    }
    
    public static Rule equals(String propertyId, Object value) {
        return new CompareRule(propertyId, value, CompareRule.CompareType.Equals);
    }
    
    public static Rule lessThan(String propertyId, Object value) {
        return new CompareRule(propertyId, value, CompareRule.CompareType.LessThan);
    }
    
    public static Rule lessOrEqual(String propertyId, Object value) {
        return new CompareRule(propertyId, value, CompareRule.CompareType.LessOrEqual);
    }
    
    public static Rule greaterThan(String propertyId, Object value) {
        return new CompareRule(propertyId, value, CompareRule.CompareType.GreaterThan);
    }
    
    public static Rule greaterOrEqual(String propertyId, Object value) {
        return new CompareRule(propertyId, value, CompareRule.CompareType.GreaterOrEqual);
    }
    
    public static Rule like(String propertyId, String value) {
        return new LikeRule(propertyId, value);
    }
    
    public static Rule like(String propertyId, String value, boolean caseSensitive) {
        return new LikeRule(propertyId, value, caseSensitive);
    }

    public static Rule isNull(String propertyId) {
        return new IsNullRule(propertyId);
    }
    
    public static Rule between(String propertyId, Object from, Object to) {
        return new BetweenRule(propertyId, from, to);
    }
    
    public static Rule contains(String propertyId, Object value) {
        return new ContainsRule(propertyId, value);
    }
    
    public static Rule in(String propertyId, List values) {
        return new InRule(propertyId, values);
    }
}
