/*
 * Apaq Framework
 * Copyright (C) 2011-2013 by Apaq
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package dk.apaq.framework.criteria.rules;


import dk.apaq.framework.criteria.Rule.FiltrationItem;
import dk.apaq.framework.criteria.Rule;
import java.util.Collection;

/**
 * Rule that retrieves the results for its subfilters and then returns a logic
 * OR for them.<br>
 * <br>
 * Example:<br>
 * Rule rule1 = ...;<br>
 * Rule rule2 = ...;<br>
 * OrRule rule = new OrRule(rule1, rule2);<br>
 * <br>
 * The example will produce a rule which will match if either rule1 OR rule2 matches.
 */
public class OrRule extends AbstractRuleCollection {

    public OrRule() {
    }

    public OrRule(Rule ... filter) {
        super(filter);
    }

    public OrRule(Collection<Rule> filters) {
        super(filters);
    }

    @Override
    public boolean passesRule(FiltrationItem item) {
        if(item == null) {
                return false;
        }

        for(Rule filter : getRules()) {
            if(filter.passesRule(item)) {
                return true;
            }
        }
        return false;
    }
}
