package dk.apaq.framework.criteria;

import dk.apaq.framework.criteria.Rules;
import dk.apaq.framework.criteria.Rule;
import dk.apaq.framework.criteria.CriteriaUtil;
import java.util.ArrayList;
import java.util.List;
import org.junit.*;
import static org.junit.Assert.*;

public class RulesTest {
    
    private List<Person> persons = new ArrayList<Person>();
    
    @Before
    public void setUp() throws Exception {
        persons.add(new Person("Michael Krog", 33, 197, true, "Micker"));
        persons.add(new Person("Hannah Krog", 30, 170, false));
        persons.add(new Person("John Doe", 1000, 181, true));
        persons.add(new Person(null, 123, 161, false));
    }

    
    /**
     * Test of and method, of class Rules.
     */
    @Test
    public void testAnd() {
        System.out.println("and");
        Rule Rule = Rules.and(Rules.like("name", "*krog", false), Rules.greaterOrEqual("height", 180));
        List<Person> result = CriteriaUtil.filterCollection(persons, Rule);
        assertEquals(1, result.size());
    }

    /**
     * Test of or method, of class Rules.
     */
    @Test
    public void testOr() {
        System.out.println("or");
        Rule Rule = Rules.or(Rules.like("name", "*krog", false), Rules.greaterOrEqual("height", 182));
        List<Person> result = CriteriaUtil.filterCollection(persons, Rule);
        assertEquals(2, result.size());
    }

    
    /**
     * Test of not method, of class Rules.
     */
    @Test
    public void testNot_String_Object() {
        System.out.println("not");
        Rule Rule = Rules.not("male", true);
        List<Person> result = CriteriaUtil.filterCollection(persons, Rule);
        assertEquals(2, result.size());
    }

    /**
     * Test of equals method, of class Rules.
     */
    @Test
    public void testEquals() {
        System.out.println("equals");
        Rule Rule = Rules.equals("male", false);
        List<Person> result = CriteriaUtil.filterCollection(persons, Rule);
        assertEquals(2, result.size());
    }

    /**
     * Test of lessThan method, of class Rules.
     */
    @Test
    public void testLessThan() {
        System.out.println("lessThan");
        Rule Rule = Rules.lessThan("height", 181);
        List<Person> result = CriteriaUtil.filterCollection(persons, Rule);
        assertEquals(2, result.size());
    }

    /**
     * Test of lessOrEqual method, of class Rules.
     */
    @Test
    public void testLessOrEqual() {
        System.out.println("lessOrEqual");
        Rule Rule = Rules.lessOrEqual("height", 181);
        List<Person> result = CriteriaUtil.filterCollection(persons, Rule);
        assertEquals(3, result.size());
    }

    /**
     * Test of greaterThan method, of class Rules.
     */
    @Test
    public void testGreaterThan() {
        System.out.println("greaterThan");
        Rule Rule = Rules.greaterThan("height", 181);
        List<Person> result = CriteriaUtil.filterCollection(persons, Rule);
        assertEquals(1, result.size());
    }

    /**
     * Test of greaterOrEqual method, of class Rules.
     */
    @Test
    public void testGreaterOrEqual() {
        System.out.println("greaterOrEqual");
        Rule Rule = Rules.greaterOrEqual("height", 181);
        List<Person> result = CriteriaUtil.filterCollection(persons, Rule);
        assertEquals(2, result.size());
    }

    /**
     * Test of like method, of class Rules.
     */
    @Test
    public void testLike_3args() {
        System.out.println("like");
        Rule Rule = Rules.like("name", "*krog", false);
        List<Person> result = CriteriaUtil.filterCollection(persons, Rule);
        assertEquals(2, result.size());
    }

    /**
     * Test of isNull method, of class Rules.
     */
    @Test
    public void testIsNull() {
        System.out.println("isNull");
        Rule Rule = Rules.isNull("name");
        List<Person> result = CriteriaUtil.filterCollection(persons, Rule);
        assertEquals(1, result.size());
    }

    /**
     * Test of between method, of class Rules.
     */
    @Test
    public void testBetween() {
        System.out.println("between");
        Rule Rule = Rules.between("age", 29, 33.4);
        List<Person> result = CriteriaUtil.filterCollection(persons, Rule);
        assertEquals(2, result.size());
    }

    /**
     * Test of contains method, of class Rules.
     */
    @Test
    public void testContains() {
        System.out.println("contains");
        Rule Rule = Rules.contains("nickNames", "Micker");
        List<Person> result = CriteriaUtil.filterCollection(persons, Rule);
        assertEquals(1, result.size());
    }
}
