package dk.apaq.framework.repository.jpa;


import dk.apaq.framework.repository.jpa.EntityManagerRepository;
import dk.apaq.framework.criteria.Limit;
import dk.apaq.framework.criteria.Rule;
import dk.apaq.framework.criteria.Sorter;
import dk.apaq.framework.criteria.rules.CompareRule;
import dk.apaq.framework.criteria.rules.OrRule;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import dk.apaq.framework.criteria.Criteria;
import dk.apaq.framework.criteria.Rules;
import dk.apaq.framework.criteria.rules.LikeRule;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author krog
 */
public class EntityManagerCrudTest {

    public EntityManagerCrudTest() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }

    @Before
    public void setUp() {
        EntityManagerFactory emf = Persistence.createEntityManagerFactory("Test");
        em = emf.createEntityManager();
        crud = new EntityManagerRepository<MockEntity, String>(em, MockEntity.class);
        crud.setTransactionsHandled(true);
    }

    @After
    public void tearDown() {
    }

    private EntityManager em;
    private EntityManagerRepository<MockEntity, String> crud;

   
    
    /**
     * Test of create method, of class EntityManagerCrud.
     */
    @Test
    public void testCreate() {
        
        System.out.println("create");
        MockEntity entity = new MockEntity();
        entity.setName("Bob");
        entity = crud.save(entity);
        MockEntity result = crud.findOne(entity.getId());
        
        assertNotNull(result);
        assertEquals("Bob", result.getName());
        assertTrue(crud.exists(entity.getId()));
        
        System.out.println("listing entities");
        for(MockEntity e : crud.findAll()) {
            System.out.println(e.getName());
        }
    }
    
    @Test
    public void testList() {
        System.out.println("list");
        MockEntity e1 = crud.save(new MockEntity());
        MockEntity e2 = crud.save(new MockEntity());
        MockEntity e3 = crud.save(new MockEntity());
        
        System.out.println("listing entities");
        for(MockEntity e : crud.findAll()) {
            System.out.println(e.getName());
        }

        e1.setName("Karl");
        e2.setName("Michael");
        e3.setName("Hannah");

        crud.save(e1);
        crud.save(e2);
        crud.save(e3);
        
        System.out.println("listing entities");
        for(MockEntity e : crud.findAll()) {
            System.out.println(e.getName());
        }

        
        Rule filter = new OrRule(new CompareRule("name", "Karl", CompareRule.CompareType.Equals),
                            new CompareRule("name", "Michael", CompareRule.CompareType.Equals),
                            new CompareRule("name", "Hannah", CompareRule.CompareType.Equals));
        Sorter sorter = new Sorter("name");
        Limit limit = new Limit(2);

        List<MockEntity> result = crud.findAll(new Criteria(filter, sorter , limit));
        assertEquals(2, result.size());

        MockEntity entity1 = crud.findOne(result.get(0).getId());
        MockEntity entity2 = crud.findOne(result.get(1).getId());
        assertEquals("Hannah", entity1.getName());
        assertEquals("Karl", entity2.getName());
        
        List<MockEntity> result2 = crud.findAll(new Criteria(filter, sorter, limit));
        assertEquals(2, result2.size());
        
        entity1 = result2.get(0);
        entity2 = result2.get(1);
        assertEquals("Hannah", entity1.getName());
        assertEquals("Karl", entity2.getName());

    }
    
    @Test
    public void testCountByCriteria() {
        MockEntity e1 = crud.save(new MockEntity());
        MockEntity e2 = crud.save(new MockEntity());
        MockEntity e3 = crud.save(new MockEntity());
        
        e1.setName("Karl");
        e2.setName("Michael");
        e3.setName("Hannah");

        crud.save(e1);
        crud.save(e2);
        crud.save(e3);
        
        Rule filter = new LikeRule("name", "Michael", false);
        
        long count = crud.count(new Criteria(filter));
        assertEquals(1, count);
    }


    /**
     * Test of update method, of class EntityManagerCrud.
     */
    @Test
    public void testUpdate() {
        System.out.println("update");
        MockEntity result = crud.save(new MockEntity());
        assertNotNull(result);
        
        String id = result.getId();

        result.setName("Test");
        crud.save(result);
        result = crud.findOne(id);
        assertEquals("Test", result.getName());
    }

    /**
     * Test of delete method, of class EntityManagerCrud.
     */
    @Test
    public void testDelete() {
        System.out.println("delete");
        MockEntity result = crud.save(new MockEntity());
        assertNotNull(result);

        String id = result.getId();
        
        crud.delete(id);
        result = crud.findOne(id);
        assertNull(result);
    }

    @Test
    public void testListManyIds() {
        int count = 1000;
        for(int i = 0; i<count;i++) {
            crud.save(new MockEntity());
        }

        List<MockEntity> idlist = crud.findAll(new Criteria(new Limit(500,500)));
        assertTrue(idlist.size() == 500);
    }
    
    @Test
    public void testWickedCriteria() {
        Rule rule1 = Rules.or();
        Rule rule2 = Rules.and(Rules.equals("name", "test"));
        Rule rule3 = Rules.or(rule1, rule2);
        Rule rule4 = Rules.not(rule3);
        Rule rule5 = Rules.or(rule3, rule4);
        
        List<MockEntity> idlist = crud.findAll(new Criteria(rule5));
        assertNotNull(idlist);
    }

}