package dk.apaq.framework.repository.mongo;

import java.io.Serializable;
import java.util.Arrays;
import java.util.List;

import dk.apaq.framework.criteria.Criteria;
import dk.apaq.framework.criteria.Limit;
import dk.apaq.framework.criteria.mongo.CriteriaForMongoDb;
import dk.apaq.framework.repository.Repository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.repository.query.MongoEntityInformation;
import org.springframework.data.mongodb.repository.support.MongoRepositoryFactory;
import org.springframework.data.mongodb.repository.support.SimpleMongoRepository;

/**
 *
 * @author krog
 */
public class MongodbRepository<BEANTYPE, IDTYPE extends Serializable>
        extends SimpleMongoRepository<BEANTYPE, IDTYPE>
        implements Repository<BEANTYPE, IDTYPE> {

    private static final Logger LOG = LoggerFactory.getLogger(MongodbRepository.class);
    private final Class<BEANTYPE> clazz;
    private MongoOperations mongoOperations;

    public MongodbRepository(MongoRepositoryFactory repositoryFactory, MongoOperations mongoOperations, Class<BEANTYPE> clazz) {
        super((MongoEntityInformation<BEANTYPE, IDTYPE>) repositoryFactory.getEntityInformation(clazz), mongoOperations);
        this.mongoOperations = mongoOperations;
        this.clazz = clazz;
        if (clazz == null) {
            throw new NullPointerException("Class was null");
        }

    }

    public BEANTYPE findOne(dk.apaq.framework.criteria.Criteria criteria) {
        criteria = criteria == null ? new dk.apaq.framework.criteria.Criteria() : criteria;
        Limit limit = criteria.getLimit() == null ? new Limit(0, 1) : new Limit(criteria.getLimit().getOffset(), 1);
        criteria = new dk.apaq.framework.criteria.Criteria(criteria.getRule(), criteria.getSorter(), limit);

        List<BEANTYPE> result = findAll(criteria);
        return result.isEmpty() ? null : result.get(0);
    }

    public List<BEANTYPE> findAll(IDTYPE... ids) {
        return (List<BEANTYPE>) findAll(Arrays.asList(ids));
    }

    public List<BEANTYPE> findAll(dk.apaq.framework.criteria.Criteria criteria) {
        Query q = CriteriaForMongoDb.translate(clazz, criteria);
        return mongoOperations.find(q, clazz);
    }

    public long count(Criteria criteria) {
        Query q = CriteriaForMongoDb.translate(clazz, criteria);
        return mongoOperations.count(q, getEntityInformation().getCollectionName());
    }

}
