package dk.apaq.framework.repository;

import dk.apaq.framework.repository.CollectionRepository;
import dk.apaq.framework.criteria.Limit;
import dk.apaq.framework.criteria.Sorter;
import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import dk.apaq.framework.criteria.Criteria;
import dk.apaq.framework.criteria.Rule;
import dk.apaq.framework.criteria.rules.LikeRule;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author krog
 */
public class CollectionCrudTest {

    public CollectionCrudTest() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }

    @Before
    public void setUp() {
        List<MockEntity> list = new ArrayList<MockEntity>();
        list.add(new MockEntity("Karl", new Date()));
        list.add(new MockEntity("Michael", new Date()));
        list.add(new MockEntity("Hannah", new Date()));
        instance = new CollectionRepository<MockEntity>(list);

    }

    @After
    public void tearDown() {
    }

    private CollectionRepository<MockEntity> instance;

    /**
     * Test of read method, of class CollectionCrud.
     */
    @Test
    public void testRead() {
        System.out.println("read");


        List<MockEntity> idlist = instance.findAll((Criteria)null);

        MockEntity result1 = instance.findOne(instance.getIdFromEntity(idlist.get(1)));
        MockEntity result2 = instance.findOne(instance.getIdFromEntity(idlist.get(2)));
        assertEquals("Michael", result1.getName());
        assertEquals("Hannah", result2.getName());
        
    }

    /**
     * Test of findAllIds method, of class CollectionCrud.
     */
    @Test
    public void testfindAllIdsByCriteria() {
        System.out.println("findAllIds");
        Rule filter = new LikeRule("name", "michael", false);
        Sorter sorter = new Sorter("name");
        Limit limit = new Limit(1);

        List<String> idlist = instance.findAllIds(new Criteria(filter));
        MockEntity entity = instance.findOne(idlist.get(0));
        assertEquals("Michael", entity.getName());

        idlist = instance.findAllIds(new Criteria(sorter));
        entity = instance.findOne(idlist.get(0));
        assertEquals("Hannah", entity.getName());

        idlist = instance.findAllIds(new Criteria(limit));
        assertEquals(1, idlist.size());
    }
    
    @Test
    public void testfindAllByCriteria() {
        System.out.println("findAll");
        Rule filter = new LikeRule("name", "michael", false);
        Sorter sorter = new Sorter("name");
        Limit limit = new Limit(1);

        List<MockEntity> list = instance.findAll(new Criteria(filter));
        MockEntity entity = list.get(0);
        assertEquals("Michael", entity.getName());

        list = instance.findAll(new Criteria(sorter));
        entity = list.get(0);
        assertEquals("Hannah", entity.getName());

        list = instance.findAll(new Criteria(limit));
        assertEquals(1, list.size());
    }
    
    @Test
    public void testfindAllByIdList() {
        System.out.println("findAllIds");
        
        
        Iterable<MockEntity> list = instance.findAll();
        List<String> idlist = new ArrayList<String>();
        for(MockEntity e : list) {
            idlist.add(instance.getIdFromEntity(e));
        }
        list = instance.findAll(idlist);
        
        Iterator it = list.iterator();
        assertTrue(it.hasNext());it.next();
        assertTrue(it.hasNext());it.next();
        assertTrue(it.hasNext());it.next();
        assertFalse(it.hasNext());
    }
    
    @Test
    public void testCountByCriteria() {
        
        Rule filter = new LikeRule("name", "michael", false);
        
        long count = instance.count(new Criteria(filter));
        assertEquals(1, count);
    }
    


}